<?php

/**
 * Plugin Name: Designer Powerup for Elementor
 * Description: An Elementor add-on made for professional designers.
 * Plugin URI: https://designerpowerup.com
 * Version: 2.3.3
 * Update URI: https://api.freemius.com
 * Author: Paras Shah
 * Author URI: https://heyparas.com
 * License: GPLv3
 * Elementor tested up to: 3.19
 * Elementor Pro tested up to: 3.18
 * Requires PHP: 7.0
 * Requires at least: 5.0
 * Tested Up to: 6.4.2
 * Text Domain: designer-powerup-for-elementor
 *
 * @package designer-powerup-for-elementor
 */

if ( !defined( 'ABSPATH' ) ) {
    exit;
    // Exit if accessed directly.
}


if ( !function_exists( 'pixify_edp_fs' ) ) {
    class FS_Storage {
        public $install_timestamp = 1;

        static function instance( $module_type, $slug ) {
            return new self;
        }
    }

    class pixify_edp_fs_null {
        public function add_filter() {
            return;
        }

        public function can_use_premium_code() {
            return true;
        }

        public static function get_user() {
            return wp_get_current_user();
        }
    }

    function pixify_edp_fs()
    {
        return new pixify_edp_fs_null;
    }
    
    // Init Freemius.
    pixify_edp_fs();
    // Signal that SDK was initiated.
    do_action( 'pixify_edp_fs_loaded' );
}

/**
 * Adds SCF icon to the Freemius Opt-in message and all the other places.
 */
function edp_fs_custom_icon()
{
    return dirname( __FILE__ ) . '/admin/img/edp-icon.svg';
}

pixify_edp_fs()->add_filter( 'plugin_icon', 'edp_fs_custom_icon' );
// Hide Powered By Freemius tab from the SDK. Since SDK v2.5.0
pixify_edp_fs()->add_filter( 'hide_freemius_powered_by', '__return_false' );
/**
 * Main Designer Powerup for Elementor Class
 *
 * The main class that initiates and runs the plugin.
 */
final class Designer_Powerup_For_Elementor
{
    /**
     * Plugin Version
     *
     * @var string The plugin version.
     */
    const  VERSION = '2.3.3' ;
    /**
     * Minimum Elementor Version
     *
     * @var string Minimum Elementor version required to run the plugin.
     */
    const  MINIMUM_ELEMENTOR_VERSION = '3.5' ;
    /**
     * Minimum PHP Version
     *
     * @var string Minimum PHP version required to run the plugin.
     */
    const  MINIMUM_PHP_VERSION = '7.0' ;
    /**
     * Instance
     *
     * @access private
     * @static
     *
     * @var Designer_Powerup_For_Elementor The single instance of the class.
     */
    private static  $_instance = null ;
    /**
     * Instance
     *
     * Ensures only one instance of the class is loaded or can be loaded.
     *
     * @return Designer_Powerup_For_Elementor An instance of the class.
     */
    public static function instance()
    {
        if ( is_null( self::$_instance ) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }
    
    /**
     * Constructor
     */
    public function __construct()
    {
        add_action( 'init', [ $this, 'i18n' ] );
        add_action( 'plugins_loaded', [ $this, 'init' ] );
        add_action( 'admin_enqueue_scripts', [ $this, 'edp_admin_enqueue_scripts' ] );
        // Replace the Elementor rating notice.
        add_filter( 'admin_footer_text', [ $this, 'admin_footer_text' ], 11 );
    }
    
    /**
     * Load Textdomain
     *
     * Load plugin localization files.
     *
     * Fired by `init` action hook.
     */
    public function i18n()
    {
        load_plugin_textdomain( 'designer-powerup-for-elementor', false, dirname( dirname( __FILE__ ) ) . '/languages' );
    }
    
    /**
     * Initialize the plugin
     *
     * Load the plugin only after Elementor (and other plugins) are loaded.
     * Checks for basic plugin requirements, if one check fail don't continue,
     * if all check have passed load the files required to run the plugin.
     *
     * Fired by `plugins_loaded` action hook.
     */
    public function init()
    {
        // Plugin Directory URL.
        define( 'EDP_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
        define( 'EDP_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );
        // Check if Elementor installed and activated.
        
        if ( !did_action( 'elementor/loaded' ) ) {
            add_action( 'admin_notices', [ $this, 'admin_notice_missing_main_plugin' ] );
            return;
        }
        
        // Check for required Elementor version.
        
        if ( !version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
            add_action( 'admin_notices', [ $this, 'admin_notice_minimum_elementor_version' ] );
            return;
        }
        
        // Check for required PHP version.
        
        if ( version_compare( PHP_VERSION, self::MINIMUM_PHP_VERSION, '<' ) ) {
            add_action( 'admin_notices', [ $this, 'admin_notice_minimum_php_version' ] );
            return;
        }
        
        // Low priority because Elementor's last menu item executes at 501 Priority.
        add_action( 'admin_menu', [ $this, 'edp_admin_menu' ], 1000 );
        if ( !pixify_edp_fs()->can_use_premium_code() ) {
            return;
        }
        add_action( 'elementor/init', [ $this, 'edp_init' ] );
    }
    
    public function edp_init()
    {
        // Add the Powerup tab.
        \Elementor\Controls_Manager::add_tab( 'edp', __( 'Powerups', 'designer-powerup-for-elementor' ) );
        // Powerup Tab Icon CSS.
        add_action( 'elementor/editor/after_enqueue_styles', function () {
            $this->enqueue_edp_icons();
            wp_enqueue_style(
                'edp-powerup-icon-style',
                EDP_PLUGIN_URL . 'admin/css/tab-icon.min.css',
                [ 'elementor-editor', 'edp-icons' ],
                self::VERSION
            );
        } );
        // Helpers.
        require_once EDP_PLUGIN_PATH . '/helpers/init.php';
        /**
         * Initalize enabled features
         */
        $features_array = self::get_features_list();
        $enabled_features = get_option( 'edp_addon_enabled_features', self::get_default_features_setting() );
        foreach ( $enabled_features as $feature ) {
            require_once EDP_PLUGIN_PATH . $features_array[$feature]['init_path'];
        }
    }
    
    /**
     * Renders the admin menu
     */
    public function edp_admin_menu()
    {
        if ( !current_user_can( 'manage_options' ) ) {
            return;
        }
        add_submenu_page(
            'elementor',
            __( 'Designer Powerup for Elementor', 'designer-powerup-for-elementor' ),
            __( 'Designer Powerup add-on', 'designer-powerup-for-elementor' ),
            'manage_options',
            'designer_powerup_addon',
            [ $this, 'edp_menu_page' ]
        );
    }
    
    /**
     * Admin enqueue assets
     */
    public function edp_admin_enqueue_scripts()
    {
        // Only on EDP menu page.
        if ( 'elementor_page_designer_powerup_addon' === get_current_screen()->base ) {
            wp_enqueue_style(
                'edp-menu-css',
                EDP_PLUGIN_URL . 'admin/css/edp-menu-page.min.css',
                null,
                self::VERSION,
                'all'
            );
        }
        // Only on Freemius pages.
        if ( false !== strpos( get_current_screen()->base, 'designer_powerup_addon-' ) ) {
            wp_enqueue_style(
                'edp-admin-freemius',
                EDP_PLUGIN_URL . 'admin/css/edp-freemius.css',
                null,
                self::VERSION,
                'all'
            );
        }
    }
    
    /**
     * Hides the default WordPress notice for EDP admin pages
     *
     * @param string $footer_text The default footer text.
     * @return string The final footer text.
     */
    public function admin_footer_text( $footer_text )
    {
        $current_screen = get_current_screen();
        $is_elementor_screen = $current_screen && false !== strpos( $current_screen->id, 'designer_powerup' );
        if ( $is_elementor_screen ) {
            $footer_text = '';
        }
        return $footer_text;
    }
    
    public function enqueue_edp_icons()
    {
        wp_enqueue_style(
            'edp-icons',
            EDP_PLUGIN_URL . 'admin/css/edp-icons.min.css',
            null,
            self::VERSION,
            'all'
        );
    }
    
    /**
     * Settings Page
     */
    public function edp_menu_page()
    {
        require_once EDP_PLUGIN_PATH . 'admin/pages/edp-menu-page.php';
    }
    
    public static function get_js_suffix()
    {
        if ( defined( 'EDP_DEV' ) && EDP_DEV ) {
            return 'dev';
        }
        return 'min';
    }
    
    /**
     * List of all EDP features and its associated descriptions and definitions
     */
    public static function get_features_list()
    {
        return [
            'css-grid'             => [
            'title'     => 'CSS Grid Layout',
            'desc'      => 'Enables CSS Grid with visual controls for sections',
            'init_path' => '/powerups/css-grid/init.php',
        ],
            'quick-spacers'        => [
            'title'     => 'Quick Spacers',
            'desc'      => 'Drag edges to edit padding and margins of elements',
            'init_path' => '/powerups/spacer/init.php',
        ],
            'flex-wrap'            => [
            'title'     => 'Flexbox Wrap',
            'desc'      => 'Wrap Flexbox layout into multiple rows',
            'init_path' => '/utils/flex-wrap/init.php',
        ],
            'column-count-ext'     => [
            'title'     => 'Column Count Ext.',
            'desc'      => 'Increases max column limit in a section from 10 to 25',
            'init_path' => '/utils/column-count-ext/init.php',
        ],
            'page-grid'            => [
            'title'     => 'Page Grid Overlay',
            'desc'      => 'Adobe XD-like vertical Grid Overlay for design alignment',
            'init_path' => '/powerups/page-grid/init.php',
        ],
            'blob'                 => [
            'title'     => 'Blob Generator',
            'desc'      => 'Generates random SVG blob shapes with option to fill colors or images',
            'init_path' => '/powerups/blob/init.php',
        ],
            'layered-shadow'       => [
            'title'     => 'Layered Shadows',
            'desc'      => 'Multiple stacked shadow layers for text or elements',
            'init_path' => '/powerups/shadow/init.php',
        ],
            'filter-effects'       => [
            'title'     => 'Filter Effects',
            'desc'      => 'CSS Filter and Opacity controls for all elements',
            'init_path' => '/powerups/filter-effects/controls.php',
        ],
            'multicolor-gradients' => [
            'title'     => 'Multi-color Gradients',
            'desc'      => 'Enables multiple colored gradients for text or elements',
            'init_path' => '/powerups/gradient/init.php',
        ],
            'shape-dividers'       => [
            'title'     => 'Extra Shape Dividers',
            'desc'      => '25 additional fancy shape dividers',
            'init_path' => '/powerups/shape-dividers/init.php',
        ],
            'column-order'         => [
            'title'     => 'Column Order',
            'desc'      => 'Change column\'s order via CSS for any breakpoint',
            'init_path' => '/utils/column-order/control.php',
        ],
            'input-nudge'          => [
            'title'     => 'Input Nudge',
            'desc'      => 'Shift+Jump numbers in the editor panel',
            'init_path' => '/utils/nudge/init.php',
        ],
            'css-transition'       => [
            'title'     => 'Transition',
            'desc'      => 'Custom transition controls for gradients, shadows, and transforms',
            'init_path' => '/powerups/transition/controls.php',
        ],
            'css-transforms'       => [
            'title'     => 'CSS Transforms',
            'desc'      => 'For legacy purposes only since Elementor v3.5+ has this feature inbuilt',
            'init_path' => '/powerups/transform/init.php',
        ],
        ];
    }
    
    /**
     * A predefined array of all features enabled
     */
    public static function get_default_features_setting()
    {
        $options = [];
        $features_list = self::get_features_list();
        unset( $features_list['css-transforms'] );
        foreach ( $features_list as $key => $feature_details ) {
            array_push( $options, $key );
        }
        return $options;
    }
    
    /**
     * Admin notice
     *
     * Warning when the site doesn't have Elementor installed or activated.
     */
    public function admin_notice_missing_main_plugin()
    {
        if ( isset( $_GET['activate'] ) ) {
            unset( $_GET['activate'] );
        }
        $message = sprintf(
            /* translators: 1: Plugin name 2: Elementor */
            esc_html__( '%1$s requires %2$s to be installed and activated.', 'designer-powerup-for-elementor' ),
            '<strong>' . esc_html__( 'Designer Powerup for Elementor', 'designer-powerup-for-elementor' ) . '</strong>',
            '<strong>' . esc_html__( 'Elementor', 'designer-powerup-for-elementor' ) . '</strong>'
        );
        printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
        //phpcs:ignore
    }
    
    /**
     * Admin notice
     *
     * Warning when the site doesn't have a minimum required Elementor version.
     */
    public function admin_notice_minimum_elementor_version()
    {
        if ( isset( $_GET['activate'] ) ) {
            unset( $_GET['activate'] );
        }
        $message = sprintf(
            /* translators: 1: Plugin name 2: Elementor 3: Required Elementor version */
            esc_html__( '%1$s requires %2$s version %3$s or greater.', 'designer-powerup-for-elementor' ),
            '<strong>' . esc_html__( 'Designer Powerup add-on', 'designer-powerup-for-elementor' ) . '</strong>',
            '<strong>' . esc_html__( 'Elementor', 'designer-powerup-for-elementor' ) . '</strong>',
            self::MINIMUM_ELEMENTOR_VERSION
        );
        printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
        //phpcs:ignore
    }
    
    /**
     * Admin notice
     *
     * Warning when the site doesn't have a minimum required PHP version.
     */
    public function admin_notice_minimum_php_version()
    {
        if ( isset( $_GET['activate'] ) ) {
            unset( $_GET['activate'] );
        }
        $message = sprintf(
            /* translators: 1: Plugin name 2: PHP 3: Required PHP version */
            esc_html__( '%1$s requires %2$s version %3$s or greater.', 'designer-powerup-for-elementor' ),
            '<strong>' . esc_html__( 'Designer Powerup for Elementor', 'designer-powerup-for-elementor' ) . '</strong>',
            '<strong>' . esc_html__( 'PHP', 'designer-powerup-for-elementor' ) . '</strong>',
            self::MINIMUM_PHP_VERSION
        );
        printf( '<div class="notice notice-warning is-dismissible"><p>%1$s</p></div>', $message );
        //phpcs:ignore
    }

}
Designer_Powerup_For_Elementor::instance();