<?php
/**
 * Blob Generator Widget
 *
 * @package designer-powerup-for-elementor\powerups\blob
 * phpcs:disable WordPress.Security.EscapeOutput.OutputNotEscaped
 * phpcs:disable Generic.WhiteSpace.ScopeIndent.IncorrectExact
 */

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Utils;
use Elementor\Core\Schemes\Color;
use Elementor\Group_Control_Image_Size;

/**
 * Elementor widget that generates a blob shape on the fly.
 */
class ED_Powerup_Blob_Generator_Widget extends Widget_Base {

	public function get_name() {
		return 'edp-blob-generator';
	}

	public function get_title() {
		return __( 'Blob Generator', 'designer-powerup-for-elementor' );
	}

	public function get_icon() {
		return 'fas fa-splotch';
	}

	public function get_categories() {
		return [ 'general' ];
	}

	protected function register_controls() {

		$this->start_controls_section(
			'edp_blob_settings_section',
			[
				'label' => __( 'Settings', 'designer-powerup-for-elementor' ),
				'tab'   => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'edp_blob_absolute_position_description',
			[
				'raw'             => __( '<b>Tip:</b> Apply Inline and Absolute Positioning to use as a background visual element.', 'designer-powerup-for-elementor' ),
				'content_classes' => 'elementor-panel-alert elementor-panel-alert-warning',
				'render_type'     => 'ui',
				'type'            => Controls_Manager::RAW_HTML,
			]
		);

		$this->add_responsive_control(
			'edp_blob_size',
			[
				'label'      => __( 'Size', 'designer-powerup-for-elementor' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 2000,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 400,
				],
				'selectors' => [
					'{{WRAPPER}} .edp-blob > svg' => 'height: {{SIZE}}{{UNIT}}; width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'edp_blob_extra_points',
			[
				'label'       => __( 'Complexity', 'designer-powerup-for-elementor' ),
				'type'        => Controls_Manager::SLIDER,
				'size_units'  => [ 'px' ],
				'range'       => [
					'px' => [
						'min'  => 0,
						'max'  => 15,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 3,
				],
				'render_type' => 'none',
			]
		);

		$this->add_control(
			'edp_blob_randomness',
			[
				'label'       => __( 'Randomness', 'designer-powerup-for-elementor' ),
				'description' => __( 'Increases the variation in point position', 'designer-powerup-for-elementor' ),
				'type'        => Controls_Manager::SLIDER,
				'size_units'  => [ 'px' ],
				'range'       => [
					'px' => [
						'min'  => 0,
						'max'  => 20,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 8,
				],
				'render_type' => 'none',
			]
		);

		$this->add_control(
			'edp_blob_generate',
			[
				'type'        => Controls_Manager::BUTTON,
				'show_label'  => false,
				'label_block' => true,
				'button_type' => 'default elementor-button-success',
				'text'        => __( 'Generate Blob', 'designer-powerup-for-elementor' ),
				'event'       => 'edp:blobWidget:generate',
			]
		);

		$this->add_control(
			'edp_blob_note',
			[
				'raw'             => __( '<b>Tip:</b> You can click the generate button multiple times to continue generating organic and random blob shapes with the set contrast and complexity.', 'designer-powerup-for-elementor' ),
				'type'            => Controls_Manager::RAW_HTML,
				'content_classes' => 'elementor-control-field-description',
				'render_type'     => 'none',
			]
		);

		// Hidden control to store the generated SVG and trigger the JS render.
		$this->add_control(
			'edp_blob_svg',
			[
				'type'        => Controls_Manager::HIDDEN,
				'render_type' => 'template',
				'default'     => $this->get_default_blob_path(),
			]
		);

		$this->add_control(
			'edp_blob_svg_viewbox',
			[
				'type'        => Controls_Manager::HIDDEN,
				'render_type' => 'template',
				'default'     => '400',
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'edp_blob_fill_section',
			[
				'label' => __( 'Fill', 'designer-powerup-for-elementor' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'edp_blob_fill',
			[
				'label'   => __( 'Type', 'designer-powerup-for-elementor' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'color',
				'options' => [
					'color'    => __( 'Color', 'designer-powerup-for-elementor' ),
					'image'    => __( 'Image', 'designer-powerup-for-elementor' ),
					'gradient' => __( 'Gradient', 'designer-powerup-for-elementor' ),
				],
			]
		);

		$this->add_control(
			'edp_blob_color',
			[
				'label'  => __( 'Color', 'designer-powerup-for-elementor' ),
				'type'   => Controls_Manager::COLOR,
				'scheme' => [
					'type'  => Color::get_type(),
					'value' => Color::COLOR_1,
				],
				'default'   => '#ec576b',
				'selectors' => [
					'{{WRAPPER}} .edp-blob path' => 'fill: {{VALUE}};',
				],
				'condition' => [
					'edp_blob_fill' => 'color',
				],
			]
		);

		$this->add_control(
			'edp_blob_image',
			[
				'label'   => __( 'Choose Image', 'designer-powerup-for-elementor' ),
				'type'    => Controls_Manager::MEDIA,
				'dynamic' => [
					'active' => true,
				],
				'default' => [
					'url' => Utils::get_placeholder_image_src(),
				],
				'condition' => [
					'edp_blob_fill' => 'image',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Image_Size::get_type(),
			[
				'name'      => 'edp_blob_image', // Usage: `{name}_size` and `{name}_custom_dimension`.
				'default'   => 'medium',
				'separator' => 'none',
				'condition' => [
					'edp_blob_fill' => 'image',
				],
			]
		);

		$this->add_control(
			'edp_blob_image_fill_type',
			[
				'label'   => __( 'Size Fit', 'designer-powerup-for-elementor' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'shrink',
				'options' => [
					'shrink' => __( 'Shrink-to-fit', 'designer-powerup-for-elementor' ),
					'custom' => __( 'Custom', 'designer-powerup-for-elementor' ),
				],
				'condition' => [
					'edp_blob_fill' => 'image',
				],
			]
		);

		$this->add_control(
			'edp_blob_image_dimensions',
			[
				'label'     => __( 'Custom Size', 'designer-powerup-for-elementor' ),
				'type'      => Controls_Manager::POPOVER_TOGGLE,
				'condition' => [
					'edp_blob_fill' => 'image',
					'edp_blob_image_fill_type' => 'custom',
				],
			]
		);

		$this->start_popover();

		$this->add_control(
			'edp_blob_image_width',
			[
				'label' => __( 'Width', 'designer-powerup-for-elementor' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 2000,
						'step' => 5,
					],
					'%' => [
						'min' => 0,
						'max' => 200,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'condition' => [
					'edp_blob_image_fill_type' => 'custom',
					'edp_blob_image_dimensions' => 'yes',
				],
			]
		);

		$this->add_control(
			'edp_blob_image_height',
			[
				'label' => __( 'Height', 'designer-powerup-for-elementor' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range' => [
					'px' => [
						'min' => 0,
						'max' => 2000,
						'step' => 5,
					],
					'%' => [
						'min' => 0,
						'max' => 200,
					],
				],
				'condition' => [
					'edp_blob_image_fill_type' => 'custom',
					'edp_blob_image_dimensions' => 'yes',
				],
			]
		);

		$this->end_popover();

		$this->add_control(
			'edp_blob_image_position',
			[
				'label'     => __( 'Custom Position', 'designer-powerup-for-elementor' ),
				'type'      => Controls_Manager::POPOVER_TOGGLE,
				'condition' => [
					'edp_blob_fill' => 'image',
					'edp_blob_image_fill_type' => 'custom',
				],
			]
		);

		$this->start_popover();

		$this->add_control(
			'edp_blob_image_pos_x',
			[
				'label' => __( 'X axis', 'designer-powerup-for-elementor' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => -500,
						'max' => 500,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 0,
				],
				'condition' => [
					'edp_blob_fill' => 'image',
					'edp_blob_image_fill_type' => 'custom',
					'edp_blob_image_position' => 'yes',
				],
			]
		);

		$this->add_control(
			'edp_blob_image_pos_y',
			[
				'label' => __( 'Y axis', 'designer-powerup-for-elementor' ),
				'type' => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'size_units' => [ 'px' ],
				'range' => [
					'px' => [
						'min' => -500,
						'max' => 500,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => 'px',
					'size' => 0,
				],
				'condition' => [
					'edp_blob_fill' => 'image',
					'edp_blob_image_fill_type' => 'custom',
					'edp_blob_image_position' => 'yes',
				],
			]
		);

		$this->end_popover();

		$this->add_control(
			'edp_blob_gradient_col_1',
			[
				'label' => __( 'Color 1', 'designer-powerup-for-elementor' ),
				'type' => Controls_Manager::COLOR,
				'scheme' => [
					'type'  => Color::get_type(),
					'value' => Color::COLOR_1,
				],
				'default'   => '#ec576b',
				'condition' => [
					'edp_blob_fill' => 'gradient',
				],
			]
		);

		$this->add_control(
			'edp_blob_gradient_loc_1',
			[
				'label'      => __( 'Location', 'designer-powerup-for-elementor' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ '%' ],
				'range'      => [
					'%' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 0,
				],
				'condition' => [
					'edp_blob_fill' => 'gradient',
				],
			]
		);

		$this->add_control(
			'edp_blob_gradient_col_2',
			[
				'label' => __( 'Color 2', 'designer-powerup-for-elementor' ),
				'type' => Controls_Manager::COLOR,
				'scheme' => [
					'type'  => Color::get_type(),
					'value' => Color::COLOR_1,
				],
				'default'   => '#fa92a0',
				'condition' => [
					'edp_blob_fill' => 'gradient',
				],
			]
		);

		$this->add_control(
			'edp_blob_gradient_loc_2',
			[
				'label'      => __( 'Location', 'designer-powerup-for-elementor' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ '%' ],
				'range'      => [
					'%' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
				],
				'default' => [
					'unit' => '%',
					'size' => 100,
				],
				'condition' => [
					'edp_blob_fill' => 'gradient',
				],
			]
		);

		$this->add_control(
			'edp_blob_gradient_type',
			[
				'label'     => __( 'Type', 'designer-powerup-for-elementor' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'linear',
				'options'   => [
					'linear' => __( 'Linear', 'designer-powerup-for-elementor' ),
					'radial' => __( 'Radial', 'designer-powerup-for-elementor' ),
				],
				'condition' => [
					'edp_blob_fill' => 'gradient',
				],
			]
		);

		$this->add_control(
			'edp_blob_linear_gradient_angle',
			[
				'label'      => __( 'Angle', 'designer-powerup-for-elementor' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'deg' ],
				'range'      => [
					'deg' => [
						'min' => 0,
						'max' => 360,
						'step' => 1,
					],
				],
				'default'   => [
					'unit' => 'deg',
					'size' => 0,
				],
				'condition' => [
					'edp_blob_gradient_type' => 'linear',
					'edp_blob_fill' => 'gradient',
				],
			]
		);

		$this->add_control(
			'edp_blob_radial_gradient_cx',
			[
				'label'      => __( 'X Pos', 'designer-powerup-for-elementor' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ '%' ],
				'range'      => [
					'%' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
				],
				'default'   => [
					'unit' => 'px',
					'size' => 50,
				],
				'condition' => [
					'edp_blob_gradient_type' => 'radial',
					'edp_blob_fill' => 'gradient',
				],
			]
		);

		$this->add_control(
			'edp_blob_radial_gradient_cy',
			[
				'label'      => __( 'Y Pos', 'designer-powerup-for-elementor' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ '%' ],
				'range'      => [
					'%' => [
						'min' => 0,
						'max' => 100,
						'step' => 1,
					],
				],
				'default'   => [
					'unit' => 'px',
					'size' => 50,
				],
				'condition' => [
					'edp_blob_gradient_type' => 'radial',
					'edp_blob_fill' => 'gradient',
				],
			]
		);

		$this->end_controls_section();

	}

	protected function content_template() {
		?>

		<div class="edp-blob">
			<svg width="{{ settings.edp_blob_size.size }}" height="{{ settings.edp_blob_size.size }}" viewBox="0 0 {{ settings.edp_blob_svg_viewbox }} {{ settings.edp_blob_svg_viewbox }}" xmlns="http://www.w3.org/2000/svg">

				<#

				var model = view.getEditModel(),
					id = model.get('id');

				if ( 'image' === settings.edp_blob_fill ) {

					var image = {
						id: settings.edp_blob_image.id,
						url: settings.edp_blob_image.url,
						size: settings.edp_blob_image_size,
						dimension: settings.edp_blob_image_custom_dimension,
						model: view.getEditModel()
					};

					var image_url = elementor.imagesManager.getImageUrl( image );

					if ( 'custom' === settings.edp_blob_image_fill_type ) {
						var image_width = ( settings.edp_blob_image_dimensions && settings.edp_blob_image_width.size ) ? settings.edp_blob_image_width.size + settings.edp_blob_image_width.unit : '100%';
						var image_height = ( settings.edp_blob_image_dimensions && settings.edp_blob_image_height.size ) ? settings.edp_blob_image_height.size + settings.edp_blob_image_height.unit : '100%';
						var image_pos_x = ( settings.edp_blob_image_position ) ? settings.edp_blob_image_pos_x.size : 0;
						var image_pos_y = ( settings.edp_blob_image_position ) ? settings.edp_blob_image_pos_y.size : 0;
						#>
						<defs>
							<pattern id="image_{{id}}" width="1" height="1" patternContentUnits="userSpaceOnUse">
								<image xlink:href="{{ image_url }}" width="{{ image_width }}" height="{{ image_height }}" x="{{ image_pos_x }}px" y="{{ image_pos_y }}px" />
							</pattern>
						</defs>
						<#
					} else {
						#>
						<defs>
							<pattern id="image_{{id}}" width="1" height="1" patternContentUnits="objectBoundingBox">
								<image xlink:href="{{ image_url }}" width="1" height="1" preserveAspectRatio="none" />
							</pattern>
						</defs>
						<#
					}

				} else if ( 'gradient' === settings.edp_blob_fill ) {
					if ( 'linear' === settings.edp_blob_gradient_type ) {
						#>
						<defs>
							<linearGradient id="gradient_{{id}}" gradientUnits="objectBoundingBox" gradientTransform="rotate({{settings.edp_blob_linear_gradient_angle.size}})">
								<stop offset="{{settings.edp_blob_gradient_loc_1.size}}%" stop-color="{{settings.edp_blob_gradient_col_1}}"/>
								<stop offset="{{settings.edp_blob_gradient_loc_2.size}}%" stop-color="{{settings.edp_blob_gradient_col_2}}"/>
							</linearGradient>
						</defs>
						<#
					} else {
						#>
						<defs>
							<radialGradient id="gradient_{{id}}" gradientUnits="objectBoundingBox" cx="{{settings.edp_blob_radial_gradient_cx.size}}%" cy="{{settings.edp_blob_radial_gradient_cy.size}}%">
								<stop offset="{{settings.edp_blob_gradient_loc_1.size}}%" stop-color="{{settings.edp_blob_gradient_col_1}}"/>
								<stop offset="{{settings.edp_blob_gradient_loc_2.size}}%" stop-color="{{settings.edp_blob_gradient_col_2}}"/>
							</radialGradient>
						</defs>
						<#
					}
				}
				#>

				{{{ settings.edp_blob_svg }}}

			</svg>
		</div>

		<?php

	}

	protected function render() {

		$id = $this->get_id();
		$settings = $this->get_settings_for_display();

		echo '<div class="edp-blob">';
			echo '<svg width="' . $settings['edp_blob_size']['size'] . '" height="' . $settings['edp_blob_size']['size'] . '" viewBox="0 0 ' . $settings['edp_blob_svg_viewbox'] . ' ' . $settings['edp_blob_svg_viewbox'] . '" xmlns="http://www.w3.org/2000/svg">';

				if ( 'image' === $settings['edp_blob_fill'] ) {
					$image_url = Group_Control_Image_Size::get_attachment_image_src( $settings['edp_blob_image']['id'], 'edp_blob_image', $settings );

					$image_width = ( $settings['edp_blob_image_dimensions'] && $settings['edp_blob_image_width']['size'] ) ? $settings['edp_blob_image_width']['size'] . $settings['edp_blob_image_width']['unit'] : '100%';
					$image_height = ( $settings['edp_blob_image_dimensions'] && $settings['edp_blob_image_height']['size'] ) ? $settings['edp_blob_image_height']['size'] . $settings['edp_blob_image_height']['unit'] : '100%';
					$image_pos_x = ( $settings['edp_blob_image_position'] ) ? $settings['edp_blob_image_pos_x']['size'] . 'px' : '0';
					$image_pos_y = ( $settings['edp_blob_image_position'] ) ? $settings['edp_blob_image_pos_y']['size'] . 'px' : '0';

					echo '<defs>';
						if ( 'custom' === $settings['edp_blob_image_fill_type'] ) {
							echo '<pattern id="image_' . $id . '" width="1" height="1" patternContentUnits="userSpaceOnUse">';
								echo '<image xlink:href="' . $image_url . '" width="' . $image_width . '" height="' . $image_height . '" x="' . $image_pos_x . '" y="' . $image_pos_y . '" />';
							echo '</pattern>';
						} else {
							echo '<pattern id="image_' . $id . '" width="1" height="1" patternContentUnits="objectBoundingBox">';
								echo '<image xlink:href="' . $image_url . '" width="1" height="1" preserveAspectRatio="none" />';
							echo '</pattern>';
						}
					echo '</defs>';
				} else if ( 'gradient' === $settings['edp_blob_fill'] ) {
					if ( 'linear' === $settings['edp_blob_gradient_type'] ) {
						echo '<defs>';
							echo '<linearGradient id="gradient_' . $id . '" gradientUnits="objectBoundingBox" gradientTransform="rotate(' . $settings['edp_blob_linear_gradient_angle']['size'] .')">';
								echo '<stop offset="' . $settings['edp_blob_gradient_loc_1']['size'] .'%" stop-color="' . $settings['edp_blob_gradient_col_1'] . '"/>';
								echo '<stop offset="' . $settings['edp_blob_gradient_loc_2']['size'] .'%" stop-color="' . $settings['edp_blob_gradient_col_2'] . '"/>';
							echo '</linearGradient>';
						echo '</defs>';
					} else {
						echo '<defs>';
							echo '<radialGradient id="gradient_' . $id . '" gradientUnits="objectBoundingBox" cx="' . $settings['edp_blob_radial_gradient_cx']['size'] .'%" cy="' . $settings['edp_blob_radial_gradient_cy']['size'] .'%">';
								echo '<stop offset="' . $settings['edp_blob_gradient_loc_1']['size'] .'%" stop-color="' . $settings['edp_blob_gradient_col_1'] . '"/>';
								echo '<stop offset="' . $settings['edp_blob_gradient_loc_2']['size'] .'%" stop-color="' . $settings['edp_blob_gradient_col_2'] . '"/>';
							echo '</radialGradient>';
						echo '</defs>';
					}
				}

				echo $settings['edp_blob_svg'];

			echo '</svg>';
		echo '</div>';

	}

	/**
	 * A default blob path with 0.2 complexity and 0.4 contrast
	 * Used when initially rendering the element.
	 *
	 * @return string The blob group path HTML.
	 */
	private function get_default_blob_path() {
		return '<path d="M200,384.3815836227603C265.0790777542987,384.1267135729392,329.5653004906703,349.5029457190737,355.7585268259659,289.92722739155096C378.7305074348496,237.67813716260082,337.992638867165,186.22674532206182,308.64380124570783,137.2744721050064C280.6120396792197,90.51901048649103,254.37269732094745,34.06268760134478,200,30.130432049433374C140.92393228290496,25.858026840355276,90.89558179096021,68.62943081927756,60.24055826387122,119.30984869185414C28.289156535752543,172.13350383485803,13.321007410784418,236.54605767395654,42.113942480049026,291.15555781043224C72.66579493827635,349.1010707278746,134.49402819476683,384.6381255233787,200,384.3815836227603"></path>';
	}

}
