<?php

if (!defined('ABSPATH')) {
	exit; // Exit if accessed directly.
}

use \Elementor\Controls_Manager;

add_action('elementor/element/column/layout/after_section_start', function ($control_stack) {

	$control_stack->add_control(
		'edp_cssg_col',
		[
			'type' => Controls_Manager::SELECT,
			'label' => __('Layout', 'designer-powerup-for-elementor'),
			'default' => 'flex',
			'options' => [
				'flex' => __('Flex Item', 'designer-powerup-for-elementor'),
				'grid'  => __('Grid Item', 'designer-powerup-for-elementor'),
			],
			'classes' => 'elementor-control-type-select--disabled',
			'render_type' => 'ui',
			'style_transfer' => false,
		]
	);

	$control_stack->add_responsive_control(
		'edp_cssg_col_position_type',
		[
			'label' => __('Positioning', 'plugin-name'),
			'description' => __('Auto means no overlap of columns. Manual means full control over positioning with overlap. <b>Use only 1 of them at a time.</b>', 'designer-powerup-for-elementor'),
			'type' => Controls_Manager::CHOOSE,
			'options' => [
				'auto' => [
					'title' => __('Auto', 'plugin-name'),
					'icon' => 'eicon-section',
				],
				'manual' => [
					'title' => __('Manual', 'plugin-name'),
					'icon' => 'eicon-edit',
				],
			],
			'default' => 'auto',
			'style_transfer' => false,
			'toggle' => 'false',
			'render_type' => 'ui',
			'condition'     => [
				'edp_cssg_col' => 'grid',
			],
		]
	);

	$breakpoints = array_reverse(array_keys(\Elementor\Plugin::$instance->breakpoints->get_active_breakpoints()));

	edp_cssg_positioning_controls($control_stack, $breakpoints);

	$control_stack->add_control(
		'separator_panel_style',
		[
			'type' => Controls_Manager::DIVIDER,
			'condition'     => [
				'edp_cssg_col' => 'grid',
			],
		]
	);
});

/**
 * New Positioning controls
 * For Elementor versions greater than 3.4
 */
function edp_cssg_positioning_controls($control_stack, $breakpoints) {

	$auto_control_device_args = edp_cssg_positioning_generate_device_args($breakpoints, 'auto');
	$manual_control_device_args = edp_cssg_positioning_generate_device_args($breakpoints, 'manual');

	$control_stack->add_responsive_control(
		'edp_cssg_col_position_auto',
		[
			'label'      => __('Auto', 'designer-powerup-for-elementor'),
			'description'  => __('Other columns will auto adjust', 'designer-powerup-for-elementor'),
			'type' => 'edp-custom-dimensions',
			'size_units' => [],
			'allowed_dimensions' => ['top', 'right'],
			'dimension_labels' => [
				'top' => 'Column Span',
				'right' => 'Row Span',
			],
			'style_transfer' => false,
			'condition' => [
				'edp_cssg_col' => 'grid',
				'edp_cssg_col_position_type' => 'auto',
			],
			'selectors' => [
				// row-start / col-start / row-end / col-end.
				'{{WRAPPER}}' => 'grid-area: span {{RIGHT}} / span {{TOP}} / span {{RIGHT}} / span {{TOP}};',
			],
			'device_args' => $auto_control_device_args,
		]
	);

	$control_stack->add_responsive_control(
		'edp_cssg_col_position_manual',
		[
			'label'      => __('Manual', 'designer-powerup-for-elementor'),
			'description'  => __('Custom grid child position. Will overlap other manually positioned columns.', 'designer-powerup-for-elementor'),
			'type' => 'edp-custom-dimensions',
			'size_units' => [],
			'dimension_labels' => [
				'top' => 'Col Start',
				'right' => 'Col Span',
				'bottom' => 'Row Start',
				'left' => 'Row Span',
			],
			'style_transfer' => false,
			'condition' => [
				'edp_cssg_col' => 'grid',
				'edp_cssg_col_position_type' => 'manual',
			],
			'selectors' => [
				// row-start / col-start / row-end / col-end.
				'{{WRAPPER}}' => 'grid-area: {{BOTTOM}} / {{TOP}} / span {{LEFT}} / span {{RIGHT}};',
			],
			'device_args' => $manual_control_device_args,
		]
	);
}

/**
 * Generate `device_args` key with condition for positioning type control
 *
 * @param array $breakpoints A list of enabled breakpoint names
 * @param string $position_value auto|manual
 */
function edp_cssg_positioning_generate_device_args($breakpoints, $position_value = 'auto') {

	// noticed since Elementor v3.7:
	// note: control setup for edit mode - breakpoint name (`desktop`, `mobile`, etc.) for condition, breakpoint key (0 - `x` integer) for device args
	// note: control setup for frontend mode - breakpoint breakpoint key (0 - `x` integer) for both condition and device args

	$device_args = [];

	// populate device args with all active breakpoint conditions
	foreach ($breakpoints as $breakpoint_key => $breakpoint_name) {

		$control_id_responsive_selector = \Elementor\Plugin::$instance->editor->is_edit_mode() ? $breakpoint_name : $breakpoint_key;

		$control_id = ('desktop' === $breakpoint_name) ? 'edp_cssg_col_position_type' : "edp_cssg_col_position_type_{$control_id_responsive_selector}";

		$device_args[$breakpoint_key] = [
			'condition' => [
				'edp_cssg_col' => 'grid',
				$control_id => $position_value
			]
		];

		switch ($position_value) {

			case 'manual':
				$device_args[$breakpoint_key]['selectors'] = [
					'{{WRAPPER}}' => 'grid-area: {{BOTTOM}} / {{TOP}} / span {{LEFT}} / span {{RIGHT}};',
				];
				break;

			case 'auto':
			default: // auto
				$device_args[$breakpoint_key]['selectors'] = [
					'{{WRAPPER}}' => 'grid-area: span {{RIGHT}} / span {{TOP}} / span {{RIGHT}} / span {{TOP}};',
				];
				break;
		}
	}

	return $device_args;
}

/**
 * Pre-set column control overrides to only show in flexbox mode
 */
add_action('elementor/element/column/layout/before_section_end', function ($control_stack) {

	// column width
	$control_stack->update_responsive_control(
		'_inline_size',
		[
			'condition' => ['edp_cssg_col' => 'flex'],
		]
	);

	// column order
	$control_stack->update_responsive_control(
		'edp_column_order',
		[
			'condition' => ['edp_cssg_col' => 'flex'],
		]
	);
}, 12);
